use chumsky::span::Span;

use crate::data::{DataError, Decimal, Root};

fn check_equal_sum(root: &Root) -> Result<(), DataError> {
    log::debug!("Checking for equal sums in monounit ledger entries...");

    for le in root.all_ledger_data() {
        let Some(tx) = le.as_transaction() else { continue };
        let Some(mono) = tx.mono_unit() else { continue };

        let net = tx
            .balances
            .iter()
            .try_fold(Decimal::ZERO, |acc, b| acc.checked_add(*b.amount));
        if net != Some(Decimal::ZERO) {
            let report = ariadne::Report::build(ariadne::ReportKind::Error, tx.span()).with_labels(
                tx.balances.iter().map(|v| {
                    let span = v.amount.span().union(v.unit.span());
                    ariadne::Label::new(span).with_message("balance here")
                }),
            );

            let report = if let Some(net) = net {
                report
                    .with_message("imbalanced transaction")
                    .with_note(format!("net value: {net} {mono}"))
            } else {
                report.with_note("numeric overflow")
            };

            return Err(report.finish().into());
        }
    }

    Ok(())
}

pub fn run_checks(root: &mut Root) -> Result<(), DataError> {
    check_equal_sum(root)?;
    Ok(())
}
